/* LKM p/ detectar LKMs que so carregados no sistema.
 * Objetivo: Detectar LKM Oculto como o Sebek a partir
 * de seu carregamento.
 * Desenvolvido por Nash Leon. 
 * nashleon@yahoo.com.br
 *
 * Thanks Skynet45 for apresentation about honeypots.
 * Clube dos Mercenarios.
 * http://cdm.frontthescene.com.br/
 */

/* +  Compilacao:
 *
 * # gcc -c modlog.c -I /lib/modules/`uname -r`/build/include
 * Exemplo:
 * # gcc -c modlog.c -I /lib/modules/2.4.20-4GB-athlon/build/include/
 * 
 * + Carregamento:
 * Acrescente uma chamada ao sistema inicial de carregamento de LKMs.
 * Exemplo:
 * No /etc/modules.conf coloque na primeira linha um chamada ao modlog.
 */


#define __KERNEL__
#define MODULE

#include <linux/kernel.h>
#include <linux/module.h>
#include <linux/unistd.h>
#include <asm/uaccess.h>



/* Abaixo chamada para a tabela de syscalls */

extern void* sys_call_table[];

/* Interceptaremos apenas um system call que eh o create_module.
 * Poderiamos trabalhar em cima do cleanup tambem, no caso especifico
 * contra o Sebek.
 */

int (*orig_create_module)(char*, unsigned long);


/* Abaixo apenas imprimimos os nomes do modulos que sao
 * carregados no sistemas. Carregamos o LKM no init e depois eh soh 
 * comparar com lsmod e ver se tem algum oculto.
 */
 
int hacked_create_module(char *name, unsigned long size){
printk(KERN_NOTICE "Nome do Mdulo: %s!\n", name);
return  orig_create_module(name,size);
}

/* Estou gerando printk, mas podemos tornar isso menos
 * chamativo, sem problemas.
 */

int init_module(void){
  
printk("Carregando ModLog... ");
orig_create_module=sys_call_table[__NR_create_module];
sys_call_table[__NR_create_module]=hacked_create_module;
printk("OK!\n");
return 0;
}


void cleanup_module(void){

printk("Descarregando ModLog....");
sys_call_table[__NR_create_module]=orig_create_module;
printk("OK!\n");
}

/* Dados do LKM */

MODULE_AUTHOR("Nash Leon");
MODULE_DESCRIPTION("LKM p/ 2.4.x que visa logar em dmesg o Sebek!!");
MODULE_LICENSE("GPL");

